//=============================================================================
// KMS_SpiralEncount.js
//=============================================================================

/*
 * This plugin can be used in the environment which supports WebGL.
 */

/*:
 * @plugindesc
 * [v0.2+] Applies spiral encounter effect.(for PIXI4)
 * 
 * @author TOMY (Kamesoft), Sasuke KANNAZUKI
 *
 * @param Speed
 * @default 0.2
 * @desc Effect speed.
 *
 * @param Radius
 * @default 1.5
 * @desc Effect range. 1.0 is same as screen size.
 *
 * @param Show characters
 * @default 0
 * @desc
 * Show characters or not.
 * 0: hide, 1: show
 *
 * @help This plugin does not provide plugin commands.
 *
 * This plugin is for MV Ver1.3.0 or later.
 * Some process is added to adapt PIXI v4(by Sasuke KANNAZUKI).
 * Sasuke thanks to TOMY (Kamesoft).
 */

/*:ja
 * @plugindesc
 * [v0.2+] エンカウント時に画面を回転させるエフェクトを適用します。
 * 
 * @author TOMY (Kamesoft), 神無月サスケ
 *
 * @param Speed
 * @default 0.2
 * @desc 回転の速度です。
 *
 * @param Radius
 * @default 1.5
 * @desc エフェクトの適用半径です。1.0 で画面サイズ相当です。
 *
 * @param Show characters
 * @default 0
 * @desc
 * エフェクト時にキャラクターを表示するか指定します。
 * 0: 非表示, 1: 表示
 *
 * @help このプラグインには、プラグインコマンドはありません。
 *
 * このプラグインは MV の バージョン 1.3.0 以降で有効です。
 * 使用されている PIXI ver4 のために処理を追加しました（by 神無月サスケ）。
 * オリジナルを作られた TOMY (Kamesoft)さんに感謝します。
 */

(function()
{
  // --------------------------------------------
  // routine for PIXI4 (make original twist filter)
  // --------------------------------------------
  PIXI.TwistFilter = function() {
    var vertexSrc = [
      '#define GLSLIFY 1',

      'attribute vec2 aVertexPosition;',
      'attribute vec2 aTextureCoord;',

      'uniform mat3 projectionMatrix;',

      'varying vec2 vTextureCoord;',

      'void main(void){',
      '    gl_Position = vec4((projectionMatrix * vec3(aVertexPosition, 1.0)).xy, 0.0, 1.0);',
      '    vTextureCoord = aTextureCoord;',
      '}'
    ].join('\n');

    var fragmentSrc = [
      '#define GLSLIFY 1',
      'precision mediump float;',
      'varying vec2 vTextureCoord;',
      'varying vec4 vColor;',
      'uniform vec4 dimensions;',

      'uniform sampler2D uSampler;',
      'uniform float radius;',
      'uniform float angle;',
      'uniform vec2 offset;',

      'void main(void) {',
      '   vec2 coord = vTextureCoord - offset;',
      '   float distance = length(coord);',
      '   if (distance < radius) {',
      '       float ratio = (radius - distance) / radius;',
      '       float angleMod = ratio * ratio * angle;',
      '       float s = sin(angleMod);',
      '       float c = cos(angleMod);',
      '       coord = vec2(coord.x * c - coord.y * s, coord.x * s + coord.y * c);',
      '   }',
      '   gl_FragColor = texture2D(uSampler, coord+offset);',
      '}'
    ].join('\n');

    PIXI.Filter.call( this, vertexSrc, fragmentSrc );

    this.padding = 512;
    this.enabled = true;
    this.resolution = 1;

    // set the uniforms
    this.uniforms = this.uniforms || {};
    this.uniforms.radius = 0.5;
    this.uniforms.angle = 5.0;
    this.uniforms.offset = {x:0.5, y:0.5};
  };


  PIXI.TwistFilter.prototype = Object.create( PIXI.Filter.prototype );
  PIXI.TwistFilter.prototype.constructor = PIXI.TwistFilter;

  Object.defineProperty(PIXI.TwistFilter.prototype, 'offset', {
    get: function() {
      return this.uniforms.offset;
    },
    set: function(value) {
      this.uniforms.offset = value;
    }
  });

  Object.defineProperty(PIXI.TwistFilter.prototype, 'radius', {
    get: function() {
      return this.uniforms.radius;
    },
    set: function(value) {
      this.uniforms.radius = value;
    }
  });

  Object.defineProperty(PIXI.TwistFilter.prototype, 'angle', {
    get: function() {
      return this.uniforms.angle;
    },
    set: function(value) {
      this.uniforms.angle = value;
    }
  });

// --------------------------------------------
// 以下、本体(original routine follows)
// --------------------------------------------
if (!Graphics.hasWebGL())
{
    console.error("WebGL をサポートしていないため、フィルターを使用できません。");
    return;
}

var pluginParams = PluginManager.parameters('KMS_SpiralEncount');
var Params = {};
Params.angleSpeed = Number(pluginParams['Speed']  || 0.2);
Params.radius     = Number(pluginParams['Radius'] || 1.5);
Params.showCharacters = Boolean(Number(pluginParams['Show characters'] || 0));

if (Params.showCharacters)
{
    Scene_Map.prototype.startEncounterEffect = function()
    {
        this._encounterEffectDuration = this.encounterEffectSpeed();
    };
}

/*
 * 回転エンカウントエフェクトの適用
 */
Scene_Map.prototype.applySpiralEncounterEffect = function()
{
    this._encounterFilter = new PIXI.TwistFilter();
    this._encounterFilter.angle  = 0;
    this._encounterFilter.radius = Params.radius;

    if (this._spriteset.filters instanceof Array)
    {
        // filters に再代入しないとフィルターが変更されない
        var newFilters = this._spriteset.filters;
        newFilters.push(this._encounterFilter);
        this._spriteset.filters = newFilters;
    }
    else
    {
        this._spriteset.filters = [this._encounterFilter];
    }

    var margin = 48;
    var width  = Graphics.width;
    var height = Graphics.height;
    this._spriteset.filterArea = new Rectangle(-margin, -margin, width, height);
};

var _KMS_SpiralEncount_Scene_Map_updateEncounterEffect = Scene_Map.prototype.updateEncounterEffect;
Scene_Map.prototype.updateEncounterEffect = function()
{
    var needEffectUpdate = this._encounterEffectDuration > 0;

    _KMS_SpiralEncount_Scene_Map_updateEncounterEffect.call(this);

    if (needEffectUpdate)
    {
        var speed = this.encounterEffectSpeed();
        var n = speed - this._encounterEffectDuration;
        if (n === 2 && Graphics.isWebGL())
        {
            this.applySpiralEncounterEffect();
        }

        if (this._encounterFilter)
        {
            this._encounterFilter.angle += Params.angleSpeed;
        }
    }
};

})();
